@@ -164,6 +164,10 @@ func (win *win) renew(w, h, x, y int) {
 	win.w, win.h, win.x, win.y = w, h, x, y
 }
 
+// printLength returns the display width of s in terminal cells.
+//
+// It ignores supported terminal control sequences (see [readTermSequence])
+// and accounts for tab expansions using the `tabstop` option.
 func printLength(s string) int {
 	ind := 0
 	off := 0
@@ -244,21 +248,16 @@ func (win *win) printRight(screen tcell.Screen, y int, st tcell.Style, s string)
 }
 
 func (win *win) printReg(screen tcell.Screen, reg *reg, previewLoading bool, sxs *sixelScreen) {
-	if reg == nil {
-		return
-	}
-
-	st := tcell.StyleDefault
-
-	if reg.loading {
+	switch {
+	case reg.loading:
 		if previewLoading {
-			st = st.Reverse(true)
+			st := tcell.StyleDefault.Reverse(true)
 			win.print(screen, 2, 0, st, "loading...")
 		}
-		return
-	}
-
-	if !reg.sixel {
+	case reg.sixel:
+		sxs.printSixel(win, screen, reg)
+	default:
+		st := tcell.StyleDefault
 		for i, l := range reg.lines {
 			if i > win.h-1 {
 				break
@@ -268,7 +267,9 @@ func (win *win) printReg(screen tcell.Screen, reg *reg, previewLoading bool, sxs
 		}
 	}
 
-	sxs.printSixel(win, screen, reg)
+	if !reg.sixel {
+		sxs.lastFile = ""
+	}
 }
 
 var gThisYear = time.Now().Year()
@@ -280,7 +281,7 @@ func infotimefmt(t time.Time) string {
 	return t.Format(gOpts.infotimefmtold)
 }
 
-func fileInfo(f *file, d *dir, userWidth int, groupWidth int, customWidth int) (string, string, int) {
+func fileInfo(f *file, d *dir, userWidth, groupWidth, customWidth int) (string, string, int) {
 	var info strings.Builder
 	var custom string
 	var off int
@@ -290,25 +291,23 @@ func fileInfo(f *file, d *dir, userWidth int, groupWidth int, customWidth int) (
 		case "size":
 			if f.IsDir() && getDirCounts(d.path) {
 				switch {
-				case f.dirCount < -1:
-					info.WriteString("     !")
-				case f.dirCount < 0:
+				case f.dirCount == nil:
 					info.WriteString("     ?")
-				case f.dirCount < 10000:
-					fmt.Fprintf(&info, " %5d", f.dirCount)
+				case *f.dirCount < 10000:
+					fmt.Fprintf(&info, " %5d", *f.dirCount)
 				default:
 					info.WriteString(" 9999+")
 				}
-				continue
-			}
-
-			var sz string
-			if f.IsDir() && f.dirSize < 0 {
-				sz = "-"
 			} else {
-				sz = humanize(uint64(f.TotalSize()))
+				switch {
+				case f.dirSize != nil:
+					fmt.Fprintf(&info, " %5s", humanize(*f.dirSize))
+				case f.IsDir():
+					info.WriteString("     -")
+				default:
+					fmt.Fprintf(&info, " %5s", humanize(uint64(f.Size())))
+				}
 			}
-			fmt.Fprintf(&info, " %5s", sz)
 		case "time":
 			fmt.Fprintf(&info, " %*s", max(len(gOpts.infotimefmtnew), len(gOpts.infotimefmtold)), infotimefmt(f.ModTime()))
 		case "atime":
@@ -347,12 +346,13 @@ type dirContext struct {
 	tags       map[string]string
 }
 
+// dirRole describes what kind of directory pane is being drawn.
 type dirRole byte
 
 const (
-	Active dirRole = iota
-	Parent
-	Preview
+	Active  dirRole = iota // Current directory pane.
+	Parent                 // Parent or ancestor directory pane.
+	Preview                // Preview pane when it shows a directory listing.
 )
 
 type dirStyle struct {
@@ -394,17 +394,15 @@ func (win *win) printDir(ui *ui, dir *dir, context *dirContext, dirStyle *dirSty
 
 	if dirStyle.role == Active && (gOpts.number || gOpts.relativenumber) {
 		lnwidth = 1
-		if gOpts.number && gOpts.relativenumber {
-			lnwidth++
-		}
 		for j := 10; j <= fileslen; j *= 10 {
 			lnwidth++
 		}
+		if gOpts.number && gOpts.relativenumber {
+			lnwidth = max(lnwidth, 2)
+		}
 	}
 
-	var userWidth int
-	var groupWidth int
-	var customWidth int
+	var userWidth, groupWidth, customWidth int
 	var fetchedCustom bool
 
 	// Only fetch user/group/custom widths if configured to display them
@@ -424,6 +422,12 @@ func (win *win) printDir(ui *ui, dir *dir, context *dirContext, dirStyle *dirSty
 		}
 	}
 
+	indOff, tagOff, nameOff := lnwidth, lnwidth, lnwidth+1
+	if !gOpts.mergeindicators {
+		tagOff++
+		nameOff++
+	}
+
 	visualSelections := dir.visualSelections()
 	for i, f := range dir.files[beg:end] {
 		st := dirStyle.colors.get(f)
@@ -440,7 +444,7 @@ func (win *win) printDir(ui *ui, dir *dir, context *dirContext, dirStyle *dirSty
 				case i > dir.pos:
 					ln = fmt.Sprintf("%*d", lnwidth, i-dir.pos)
 				case gOpts.number:
-					ln = fmt.Sprintf("%*d ", lnwidth-1, i+1+beg)
+					ln = fmt.Sprintf("%-*d", lnwidth, i+1+beg)
 				default:
 					ln = fmt.Sprintf("%*d", lnwidth, 0)
 				}
@@ -463,22 +467,23 @@ func (win *win) printDir(ui *ui, dir *dir, context *dirContext, dirStyle *dirSty
 				fmtStr = gOpts.cutfmt
 			}
 		}
-		if fmtStr != "" {
-			win.print(ui.screen, lnwidth, i, parseEscapeSequence(fmtStr), " ")
-		}
-
-		// make space for select marker, and leave another space at the end
-		maxWidth := win.w - lnwidth - 2
-		// make extra space to separate windows if drawbox is not enabled
-		if !gOpts.drawbox {
-			maxWidth--
-		}
 
 		tag := " "
 		if val, ok := context.tags[path]; ok && len(val) > 0 {
 			tag = val
 		}
 
+		if fmtStr != "" {
+			ind := " "
+			if gOpts.mergeindicators {
+				ind = tag
+			}
+			win.print(ui.screen, indOff, i, parseEscapeSequence(fmtStr), ind)
+		}
+
+		// make space for select marker, and leave another space at the end
+		maxWidth := win.w - lnwidth - 2
+
 		var icon []rune
 		var iconDef iconDef
 		if gOpts.icons {
@@ -486,10 +491,14 @@ func (win *win) printDir(ui *ui, dir *dir, context *dirContext, dirStyle *dirSty
 			icon = slices.Concat([]rune(iconDef.icon), []rune{' '})
 		}
 
-		// subtract space for tag and icon
-		maxFilenameWidth := maxWidth - 1 - runeSliceWidth(icon)
+		// subtract space for icon
+		maxFilenameWidth := maxWidth - runeSliceWidth(icon)
+		// subtract space for tag if not merged with selection marker
+		if !gOpts.mergeindicators {
+			maxFilenameWidth--
+		}
 
-		info, custom, off := fileInfo(f, dir, userWidth, groupWidth, customWidth)
+		info, custom, customOff := fileInfo(f, dir, userWidth, groupWidth, customWidth)
 		infolen := len(info)
 		showInfo := infolen > 0 && 2*infolen < maxWidth
 		if showInfo {
@@ -503,7 +512,7 @@ func (win *win) printDir(ui *ui, dir *dir, context *dirContext, dirStyle *dirSty
 
 		if showInfo {
 			filename = append(filename, []rune(info)...)
-			off += lnwidth + 2 + runeSliceWidth(icon) + maxFilenameWidth
+			customOff += nameOff + runeSliceWidth(icon) + maxFilenameWidth
 		}
 
 		if i == dir.pos {
@@ -518,21 +527,25 @@ func (win *win) printDir(ui *ui, dir *dir, context *dirContext, dirStyle *dirSty
 			}
 
 			// print tag separately as it can contain color escape sequences
-			win.print(ui.screen, lnwidth+1, i, st, fmt.Sprintf(cursorFmt, tag))
+			if !gOpts.mergeindicators || fmtStr == "" {
+				win.print(ui.screen, tagOff, i, st, fmt.Sprintf(cursorFmt, tag))
+			}
 
 			line := slices.Concat(icon, filename, []rune{' '})
-			win.print(ui.screen, lnwidth+2, i, st, fmt.Sprintf(cursorFmt, string(line)))
+			win.print(ui.screen, nameOff, i, st, fmt.Sprintf(cursorFmt, string(line)))
 
 			// print over the empty space we reserved for the custom info
 			if showInfo && custom != "" {
-				win.print(ui.screen, off, i, st, fmt.Sprintf(cursorFmt, stripTermSequence(custom)))
+				win.print(ui.screen, customOff, i, st, fmt.Sprintf(cursorFmt, stripTermSequence(custom)))
 			}
 		} else {
-			if tag == " " {
-				win.print(ui.screen, lnwidth+1, i, st, " ")
-			} else {
-				tagStr := fmt.Sprintf(optionToFmtstr(gOpts.tagfmt), tag)
-				win.print(ui.screen, lnwidth+1, i, tcell.StyleDefault, tagStr)
+			if !gOpts.mergeindicators || fmtStr == "" {
+				if tag == " " {
+					win.print(ui.screen, tagOff, i, st, " ")
+				} else {
+					tagStr := fmt.Sprintf(optionToFmtstr(gOpts.tagfmt), tag)
+					win.print(ui.screen, tagOff, i, tcell.StyleDefault, tagStr)
+				}
 			}
 
 			if len(icon) > 0 {
@@ -540,21 +553,21 @@ func (win *win) printDir(ui *ui, dir *dir, context *dirContext, dirStyle *dirSty
 				if iconDef.hasStyle {
 					iconStyle = iconDef.style
 				}
-				win.print(ui.screen, lnwidth+2, i, iconStyle, string(icon))
+				win.print(ui.screen, nameOff, i, iconStyle, string(icon))
 			}
 
 			line := slices.Concat(filename, []rune{' '})
-			win.print(ui.screen, lnwidth+2+runeSliceWidth(icon), i, st, string(line))
+			win.print(ui.screen, nameOff+runeSliceWidth(icon), i, st, string(line))
 
 			// print over the empty space we reserved for the custom info
 			if showInfo && custom != "" {
-				win.print(ui.screen, off, i, st, custom)
+				win.print(ui.screen, customOff, i, st, custom)
 			}
 		}
 	}
 }
 
-func getUserWidth(dir *dir, beg int, end int) int {
+func getUserWidth(dir *dir, beg, end int) int {
 	maxw := 0
 
 	for _, f := range dir.files[beg:end] {
@@ -564,7 +577,7 @@ func getUserWidth(dir *dir, beg int, end int) int {
 	return maxw
 }
 
-func getGroupWidth(dir *dir, beg int, end int) int {
+func getGroupWidth(dir *dir, beg, end int) int {
 	maxw := 0
 
 	for _, f := range dir.files[beg:end] {
@@ -574,7 +587,7 @@ func getGroupWidth(dir *dir, beg int, end int) int {
 	return maxw
 }
 
-func getCustomWidth(dir *dir, beg int, end int) int {
+func getCustomWidth(dir *dir, beg, end int) int {
 	maxw := 0
 
 	for _, f := range dir.files[beg:end] {
@@ -584,45 +597,23 @@ func getCustomWidth(dir *dir, beg int, end int) int {
 	return maxw
 }
 
-func getWidths(wtot int) []int {
-	rsum := 0
-	for _, r := range gOpts.ratios {
-		rsum += r
-	}
-
-	wlen := len(gOpts.ratios)
-	widths := make([]int, wlen)
-
-	if gOpts.drawbox {
-		wtot -= (wlen + 1)
-	}
-
-	wsum := 0
-	for i := range wlen - 1 {
-		widths[i] = gOpts.ratios[i] * wtot / rsum
-		wsum += widths[i]
-	}
-	widths[wlen-1] = wtot - wsum
-
-	return widths
-}
-
 func getWins(screen tcell.Screen) []*win {
 	wtot, htot := screen.Size()
 
-	widths := getWidths(wtot)
+	h := max(htot-2, 0)
+	x := 0
+	y := 1
+	if gOpts.drawbox {
+		h = max(htot-4, 0)
+		x = 1
+		y = 2
+	}
 
-	wacc := 0
-	wlen := len(widths)
-	wins := make([]*win, 0, wlen)
-	for i := range wlen {
-		if gOpts.drawbox {
-			wacc++
-			wins = append(wins, newWin(widths[i], max(htot-4, 0), wacc, 2))
-		} else {
-			wins = append(wins, newWin(widths[i], max(htot-2, 0), wacc, 1))
-		}
-		wacc += widths[i]
+	widths := getWidths(wtot, gOpts.ratios, gOpts.drawbox)
+	wins := make([]*win, 0, len(widths))
+	for _, w := range widths {
+		wins = append(wins, newWin(w, h, x, y))
+		x += w + 1
 	}
 
 	return wins
@@ -642,8 +633,6 @@ type ui struct {
 	msgWin      *win
 	menuWin     *win
 	msg         string
-	regPrev     *reg
-	dirPrev     *dir
 	exprChan    chan expr
 	keyChan     chan string
 	tevChan     chan tcell.Event
@@ -683,7 +672,7 @@ func newUI(screen tcell.Screen) *ui {
 		currentFile: "",
 		sxScreen:    sixelScreen{},
 	}
-	ui.ruler, ui.rulerErr = parseRuler()
+	ui.ruler, ui.rulerErr = parseRuler(gOpts.rulerfile)
 
 	go ui.pollEvents()
 
@@ -721,15 +710,6 @@ func (ui *ui) renew() {
 	ui.menuWin.renew(wtot, 1, 0, htot-2)
 }
 
-func (ui *ui) sort() {
-	if ui.dirPrev == nil {
-		return
-	}
-	name := ui.dirPrev.name()
-	ui.dirPrev.sort()
-	ui.dirPrev.sel(name, ui.wins[0].h)
-}
-
 func (ui *ui) echo(msg string) {
 	ui.msg = msg
 }
@@ -748,9 +728,12 @@ func (ui *ui) echoerrf(format string, a ...any) {
 	ui.echoerr(fmt.Sprintf(format, a...))
 }
 
-// This represents the preview for a regular file.
+// reg represents the preview for a file.
 // This can also be used to represent the preview of a directory if
 // `dirpreviews` is enabled.
+//
+// Note: the name `reg` is historical. It originally meant "regular"
+// file preview, but `previewer` now also supports non-regular files.
 type reg struct {
 	loading  bool
 	volatile bool
@@ -761,12 +744,8 @@ type reg struct {
 }
 
 func (ui *ui) loadFile(app *app, volatile bool) {
-	if !app.nav.init {
-		return
-	}
-
-	curr, err := app.nav.currFile()
-	if err != nil {
+	curr := app.nav.currFile()
+	if curr == nil {
 		return
 	}
 
@@ -784,9 +763,10 @@ func (ui *ui) loadFile(app *app, volatile bool) {
 	}
 
 	if curr.isPreviewable() {
-		ui.regPrev = app.nav.loadReg(curr.path, volatile)
+		app.nav.loadReg(curr.path, volatile)
 	} else if curr.IsDir() {
-		ui.dirPrev = app.nav.loadDir(curr.path)
+		dir := app.nav.getDir(curr.path)
+		app.nav.checkDir(dir)
 	}
 }
 
@@ -803,8 +783,7 @@ func (ui *ui) drawPromptLine(nav *nav) {
 	sep := string(filepath.Separator)
 
 	var fname string
-	curr, err := nav.currFile()
-	if err == nil {
+	if curr := nav.currFile(); curr != nil {
 		fname = filepath.Base(curr.path)
 	}
 
@@ -851,7 +830,7 @@ func (ui *ui) drawPromptLine(nav *nav) {
 	ui.promptWin.print(ui.screen, 0, 0, st, prompt)
 }
 
-func formatRulerOpt(name string, val string) string {
+func formatRulerOpt(name, val string) string {
 	// handle escape character so it doesn't mess up the ruler
 	val = strings.ReplaceAll(val, "\033", "\033[7m\\033\033[0m")
 
@@ -869,8 +848,8 @@ func (ui *ui) drawStat(nav *nav) {
 		return
 	}
 
-	curr, err := nav.currFile()
-	if err != nil {
+	curr := nav.currFile()
+	if curr == nil {
 		return
 	}
 
@@ -881,7 +860,7 @@ func (ui *ui) drawStat(nav *nav) {
 	}
 
 	statfmt := strings.ReplaceAll(gOpts.statfmt, "|", "\x1f")
-	replace := func(s string, val string) {
+	replace := func(s, val string) {
 		if val == "" {
 			val = "\x00"
 		}
@@ -1023,19 +1002,26 @@ func (ui *ui) drawRulerFile(nav *nav) {
 	}
 
 	var stat *statData
-	curr, err := nav.currFile()
-	if err == nil {
+	curr := nav.currFile()
+	if curr != nil {
 		if curr.err == nil {
 			stat = &statData{
 				Path:        curr.path,
 				Name:        curr.Name(),
+				Extension:   curr.ext,
 				Size:        uint64(curr.Size()),
+				DirSize:     curr.dirSize,
+				DirCount:    curr.dirCount,
 				Permissions: permString(curr.Mode()),
 				ModTime:     curr.ModTime().Format(gOpts.timefmt),
+				AccessTime:  curr.accessTime.Format(gOpts.timefmt),
+				BirthTime:   curr.birthTime.Format(gOpts.timefmt),
+				ChangeTime:  curr.changeTime.Format(gOpts.timefmt),
 				LinkCount:   linkCount(curr),
 				User:        userName(curr),
 				Group:       groupName(curr),
 				Target:      curr.linkTarget,
+				CustomInfo:  curr.customInfo,
 			}
 		} else {
 			ui.echoerrf("stat: %s", curr.err)
@@ -1146,6 +1132,29 @@ func (ui *ui) drawRulerFile(nav *nav) {
 	ui.msgWin.printRight(ui.screen, 0, tcell.StyleDefault, right)
 }
 
+func (ui *ui) drawPreview(nav *nav, context *dirContext) {
+	curr := nav.currFile()
+	if curr == nil {
+		return
+	}
+
+	win := ui.wins[len(ui.wins)-1]
+	ui.sxScreen.clearSixel(win, ui.screen, curr.path)
+
+	if gOpts.preview {
+		if curr.isPreviewable() {
+			if reg, ok := nav.regCache[curr.path]; ok {
+				win.printReg(ui.screen, reg, nav.previewLoading, &ui.sxScreen)
+			}
+		} else if curr.IsDir() {
+			ui.sxScreen.lastFile = ""
+			dir := nav.getDir(curr.path)
+			dirStyle := &dirStyle{colors: ui.styles, icons: ui.icons, role: Preview}
+			win.printDir(ui, dir, context, dirStyle)
+		}
+	}
+}
+
 func (ui *ui) drawBox() {
 	st := parseEscapeSequence(gOpts.borderfmt)
 
@@ -1221,11 +1230,11 @@ func (ui *ui) dirOfWin(nav *nav, wind int) *dir {
 	if gOpts.preview {
 		wins--
 	}
-	ind := len(nav.dirs) - wins + wind
+	ind := len(nav.dirPaths) - wins + wind
 	if ind < 0 {
 		return nil
 	}
-	return nav.dirs[ind]
+	return nav.getDir(nav.dirPaths[ind])
 }
 
 func (ui *ui) draw(nav *nav) {
@@ -1253,7 +1262,7 @@ func (ui *ui) draw(nav *nav) {
 
 	switch ui.cmdPrefix {
 	case "":
-		if gOpts.rulerfile {
+		if gOpts.rulerfmt == "" {
 			ui.drawRulerFile(nav)
 		} else {
 			ui.drawStat(nav)
@@ -1275,20 +1284,7 @@ func (ui *ui) draw(nav *nav) {
 		ui.screen.ShowCursor(ui.msgWin.x+runeSliceWidth(prefix)+runeSliceWidth(left), ui.msgWin.y)
 	}
 
-	curr, err := nav.currFile()
-	if err == nil {
-		preview := ui.wins[len(ui.wins)-1]
-		ui.sxScreen.clearSixel(preview, ui.screen, curr.path)
-		if gOpts.preview {
-			if curr.isPreviewable() {
-				preview.printReg(ui.screen, ui.regPrev, nav.previewLoading, &ui.sxScreen)
-			} else if curr.IsDir() {
-				ui.sxScreen.lastFile = ""
-				preview.printDir(ui, ui.dirPrev, &context,
-					&dirStyle{colors: ui.styles, icons: ui.icons, role: Preview})
-			}
-		}
-	}
+	ui.drawPreview(nav, &context)
 
 	if gOpts.drawbox {
 		ui.drawBox()
@@ -1461,9 +1457,6 @@ func listMarks(marks map[string]string) string {
 }
 
 func listFilesInCurrDir(nav *nav) string {
-	if !nav.init {
-		return ""
-	}
 	dir := nav.currDir()
 	if dir.loading {
 		log.Printf("listFilesInCurrDir(): %s is still loading, `files` isn't ready for remote query", dir.path)
@@ -1543,7 +1536,7 @@ func addSpecialKeyModifier(val string, mod tcell.ModMask) string {
 	}
 }
 
-// This function is used to read a normal event on the client side. For keys,
+// readNormalEvent is used to read a normal event on the client side. For keys,
 // digits are interpreted as command counts but this is only done for digits
 // preceding any non-digit characters (e.g. "42y2k" as 42 times "y2k").
 func (ui *ui) readNormalEvent(ev tcell.Event, nav *nav) expr {
@@ -1690,8 +1683,8 @@ func (ui *ui) readNormalEvent(ev tcell.Event, nav *nav) expr {
 
 		var dir *dir
 		if gOpts.preview && wind == len(ui.wins)-1 {
-			curr, err := nav.currFile()
-			if err != nil {
+			curr := nav.currFile()
+			if curr == nil {
 				return nil
 			} else if !curr.IsDir() || gOpts.dirpreviews {
 				if tev.Buttons() != tcell.Button2 {
@@ -1699,7 +1692,8 @@ func (ui *ui) readNormalEvent(ev tcell.Event, nav *nav) expr {
 				}
 				return &callExpr{"open", nil, 1}
 			}
-			dir = ui.dirPrev
+
+			dir = nav.getDir(curr.path)
 		} else {
 			dir = ui.dirOfWin(nav, wind)
 			if dir == nil {
